import lib.reg as reg
from lib.parsing.parser import Parser, ParsingError
import signal
from lib.checker import dfa_transform, nfa_transform, check_task, check_alphabets, check_empty


class WebChecker():
    def __init__(self, student_string: str, task: str):
        self.student_string = student_string
        self.task = task
        self.eq = None
        self.ok = False

    def dfa_task(self, teacher_type, teacher_string, task, student_string):
        try:
            student_solution = dfa_transform(student_string, task)
            teacher_solution = dfa_transform(teacher_string, teacher_type)
        except ParsingError as ex:
            #print(ex.args)
            return("Chyba při parsování.")

        check_empty(student_solution=student_solution,
                    teacher_solution=teacher_solution)

        alphabets = check_alphabets(student_alpha=student_solution.characters,
                                    teacher_alpha=teacher_solution.characters, task=task)
        # TODO print also alphabets (TODO manage prints for all checkers better

        self.eq = reg.DFA.is_equivalent(student_solution, teacher_solution)
        task_solved = check_task(student_solution, task)
        if task_solved == "" and self.eq:
            self.ok = True

        return task_solved

    def compare(self, teacher_string: str, teacher_type: str):
        #signal.alarm(50) #
        try:
            return self.dfa_task(teacher_type=teacher_type, teacher_string=teacher_string,
                                 task=self.task, student_string=self.student_string)

        except Exception as ex:
            print("Error inside of web checker:", ex.args)


    def convert(self, student_type):
        #signal.alarm(50)

        try:
            parser = Parser()

            if self.task in {"DFA", "TOT", "MIN"}:
                automaton = dfa_transform(self.student_string, student_type)

                if self.task == "DFA":
                    return parser.dfa_to_str(automaton)
                if self.task == "TOT":
                    return parser.dfa_to_str(automaton.total())
                if self.task == "MIN":
                    return parser.dfa_to_str(automaton.minimize())
                #if canonize: TODO checkbox
                #    return parser.dfa_to_str(automaton.canonize())

            automaton = nfa_transform(self.student_string, student_type)
            if self.task in {"NFA", "EFA"}:
                if self.task == "NFA":
                    return parser.nfa_to_str(automaton.eliminate_epsilon())
                else:
                    return parser.nfa_to_str(automaton)

            if self.task == "GRA":
                print(parser.nfa_to_str(automaton))
                print(type(automaton))
                return parser.reggrammar_to_str(automaton.nfa_to_reggrammar())

            if self.task == "REG":
                return "Tohle není pěkný typ převodu. Nechcete si místo regulárního výrazu vybrat něco jiného?"
                #return parser.reggrammar_to_str(automaton.nfa_to_regex())

        except Exception as ex:
            print("Error inside of web checker:", ex.args)